// Generated by CoffeeScript 1.3.3
(function() {
  var Match, NoMatch, Parser, ParserState, Trampoline, alt, chain, check, commit, consume, debug, drop, dropPrefix, end, implicit, inspect, not_, optional, parse, reduce, regex, reject, repeat, repeatIgnore, repeatSeparated, resolve, seq, seqIgnore, setDebug, string, _parser_id,
    __slice = [].slice;

  inspect = require("util").inspect;

  debug = null;

  setDebug = function(f) {
    return debug = f;
  };

  Trampoline = (function() {

    function Trampoline() {
      this.work = [];
      this.cache = {};
    }

    Trampoline.prototype.push = function(job) {
      return this.work.push(job);
    };

    Trampoline.prototype.size = function() {
      return this.work.length;
    };

    Trampoline.prototype.next = function() {
      var f;
      f = this.work.pop();
      if (f != null) {
        return f();
      }
    };

    Trampoline.prototype.ready = function() {
      return this.work.length > 0;
    };

    Trampoline.prototype.getCache = function(parser, state) {
      var entry, x;
      x = this.cache[parser.id];
      if (!(x != null)) {
        this.cache[parser.id] = x = {};
      }
      entry = x[state.pos];
      if (!(entry != null)) {
        x[state.pos] = entry = {
          continuations: [],
          results: []
        };
      }
      return entry;
    };

    return Trampoline;

  })();

  ParserState = (function() {

    function ParserState(text, pos, end, lineno, xpos, trampoline) {
      this.text = text;
      this.pos = pos != null ? pos : 0;
      this.end = end;
      this.lineno = lineno != null ? lineno : 0;
      this.xpos = xpos != null ? xpos : 0;
      this.trampoline = trampoline != null ? trampoline : null;
      if (!(this.end != null)) {
        this.end = this.text.length;
      }
      if (!(this.trampoline != null)) {
        this.trampoline = new Trampoline();
      }
    }

    ParserState.prototype.toString = function() {
      var truncated;
      truncated = this.text.length > 10 ? "'" + this.text.slice(0, 10) + "...'" : "'" + this.text + "'";
      return "ParserState(text=" + truncated + ", pos=" + this.pos + ", end=" + this.end + ", lineno=" + this.lineno + ", xpos=" + this.xpos + ", work=" + (this.trampoline.size()) + ")";
    };

    ParserState.prototype.advance = function(n) {
      var lineno, pos, state, xpos;
      pos = this.pos;
      lineno = this.lineno;
      xpos = this.xpos;
      while (pos < this.pos + n) {
        if (this.text[pos++] === '\n') {
          lineno++;
          xpos = 0;
        } else {
          xpos++;
        }
      }
      state = new ParserState(this.text, pos, this.end, lineno, xpos, this.trampoline);
      return state;
    };

    ParserState.prototype.line = function() {
      var end, lend, lstart, text;
      text = this.text;
      end = this.end;
      lstart = this.pos;
      lend = this.pos;
      if (lstart > 0 && text[lstart] === '\n') {
        lstart--;
      }
      while (lstart > 0 && text[lstart] !== '\n') {
        lstart--;
      }
      if (text[lstart] === '\n') {
        lstart++;
      }
      while (lend < end && text[lend] !== '\n') {
        lend++;
      }
      return text.slice(lstart, lend);
    };

    ParserState.prototype.fail = function(cont, message) {
      return cont(new NoMatch(this, "Expected " + message));
    };

    ParserState.prototype.getCache = function(parser) {
      return this.trampoline.getCache(parser, this);
    };

    return ParserState;

  })();

  Match = (function() {

    function Match(state, match, commit) {
      this.state = state;
      this.match = match;
      this.commit = commit != null ? commit : false;
      this.ok = true;
    }

    Match.prototype.toString = function() {
      return "Match(state=" + this.state + ", match=" + (inspect(this.match)) + ", commit=" + this.commit + ")";
    };

    Match.prototype.equals = function(other) {
      return other.ok && this.match === other.match && this.state.pos === other.state.pos;
    };

    return Match;

  })();

  NoMatch = (function() {

    function NoMatch(state, message, abort) {
      this.state = state;
      this.message = message;
      this.abort = abort != null ? abort : false;
      this.ok = false;
    }

    NoMatch.prototype.toString = function() {
      return "NoMatch(state=" + this.state + ", message='" + this.message + "', abort=" + this.abort + ")";
    };

    NoMatch.prototype.equals = function(other) {
      return (!other.ok) && this.state.pos === other.state.pos;
    };

    return NoMatch;

  })();

  _parser_id = 0;

  Parser = (function() {

    function Parser(_message, matcher) {
      this._message = _message;
      this.matcher = matcher;
      this.id = _parser_id++;
    }

    Parser.prototype.message = function() {
      var rv;
      if (this.recursing) {
        return "...";
      }
      this.recursing = true;
      rv = typeof this._message === "function" ? this._message() : this._message;
      this.recursing = false;
      return rv;
    };

    Parser.prototype.toString = function() {
      return "Parser(" + (this.message()) + ")";
    };

    Parser.prototype.fail = function(state, cont) {
      return cont(new NoMatch(state, "Expected " + this.message()));
    };

    Parser.prototype.parse = function(state, cont) {
      var entry, newCont, r, _i, _len, _ref, _results,
        _this = this;
      if (debug != null) {
        debug("-> state=" + state);
        debug("   parse: " + this);
      }
      newCont = function(rv) {
        if (debug != null) {
          debug("<- " + rv);
        }
        return cont(rv);
      };
      entry = state.getCache(this);
      if (entry.continuations.length === 0) {
        entry.continuations.push(newCont);
        return state.trampoline.push(function() {
          return _this.matcher(state, function(rv) {
            var c, found, r, _i, _j, _len, _len1, _ref, _ref1, _results;
            found = false;
            _ref = entry.results;
            for (_i = 0, _len = _ref.length; _i < _len; _i++) {
              r = _ref[_i];
              if (r.equals(rv)) {
                found = true;
              }
            }
            if (!found) {
              if (debug != null) {
                debug("<- coming back from " + _this);
                debug("   cache++ (" + _this.id + "," + state.pos + ") = " + rv);
              }
              entry.results.push(rv);
              _ref1 = entry.continuations;
              _results = [];
              for (_j = 0, _len1 = _ref1.length; _j < _len1; _j++) {
                c = _ref1[_j];
                _results.push(c(rv));
              }
              return _results;
            }
          });
        });
      } else {
        if (debug != null) {
          debug("<- answer from cache (" + this.id + "," + state.pos + "): " + (inspect(entry.results)));
        }
        entry.continuations.push(newCont);
        _ref = entry.results;
        _results = [];
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          r = _ref[_i];
          _results.push(newCont(r));
        }
        return _results;
      }
    };

    Parser.prototype.onFail = function(newMessage) {
      var _this = this;
      return new Parser(newMessage, function(state, cont) {
        return _this.parse(state, function(rv) {
          if (rv.ok) {
            return cont(rv);
          }
          return cont(new NoMatch(rv.state, newMessage, rv.abort));
        });
      });
    };

    Parser.prototype.onMatch = function(f) {
      var _this = this;
      return new Parser(this._message, function(state, cont) {
        return _this.parse(state, function(rv) {
          if (!rv.ok) {
            return cont(rv);
          }
          rv = (function() {
            if (f instanceof Function) {
              try {
                return new Match(rv.state, f(rv.match), rv.commit);
              } catch (e) {
                return new NoMatch(state, e.toString());
              }
            } else {
              return new Match(rv.state, f, rv.commit);
            }
          })();
          return cont(rv);
        });
      });
    };

    Parser.prototype.matchIf = function(f) {
      var _this = this;
      return new Parser(this._message, function(state, cont) {
        return _this.parse(state, function(rv) {
          if (!rv.ok) {
            return cont(rv);
          }
          if (!f(rv.match)) {
            return _this.fail(state, cont);
          }
          return cont(rv);
        });
      });
    };

    Parser.prototype.then = function(p) {
      return chain(this, p, function(a, b) {
        return [a, b];
      });
    };

    Parser.prototype.or = function() {
      var others;
      others = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      return alt.apply(null, [this].concat(__slice.call(others)));
    };

    Parser.prototype.optional = function(defaultValue) {
      if (defaultValue == null) {
        defaultValue = "";
      }
      return optional(this, defaultValue);
    };

    Parser.prototype.check = function() {
      return check(this);
    };

    Parser.prototype.commit = function() {
      return commit(this);
    };

    Parser.prototype.not_ = function() {
      return not_(this);
    };

    Parser.prototype.drop = function() {
      return drop(this);
    };

    Parser.prototype.repeat = function(minCount, maxCount) {
      return repeat(this, minCount, maxCount);
    };

    Parser.prototype.times = function(count) {
      return repeat(this, count, count);
    };

    return Parser;

  })();

  end = new Parser("end", function(state, cont) {
    if (state.pos === state.end) {
      return cont(new Match(state, null));
    } else {
      return this.fail(state, cont);
    }
  });

  reject = new Parser("failure", function(state, cont) {
    return this.fail(state, cont);
  });

  string = function(s) {
    var len;
    len = s.length;
    return new Parser("'" + s + "'", function(state, cont) {
      var candidate;
      candidate = state.text.slice(state.pos, state.pos + len);
      if (candidate === s) {
        return cont(new Match(state.advance(len), candidate));
      } else {
        return state.fail(cont, this.message());
      }
    });
  };

  regex = function(r) {
    var i, m, r2, source;
    i = r.ignoreCase ? "i" : "";
    m = r.multiline ? "m" : "";
    source = r.source[0] === "^" ? r.source : "^" + r.source;
    r2 = new RegExp(source, i + m);
    return new Parser(r.toString(), function(state, cont) {
      m = r2.exec(state.text.slice(state.pos));
      if (m != null) {
        return cont(new Match(state.advance(m[0].length), m));
      } else {
        return this.fail(state, cont);
      }
    });
  };

  optional = function(p, defaultValue) {
    if (defaultValue == null) {
      defaultValue = "";
    }
    p = implicit(p);
    return new Parser((function() {
      return "optional " + (resolve(p).message());
    }), function(state, cont) {
      p = resolve(p);
      return p.parse(state, function(rv) {
        if (rv.ok) {
          return cont(rv);
        }
        return cont(new Match(state, defaultValue));
      });
    });
  };

  check = function(p) {
    p = implicit(p);
    return new Parser((function() {
      return resolve(p).message();
    }), function(state, cont) {
      p = resolve(p);
      return p.parse(state, function(rv) {
        if (!rv.ok) {
          return cont(rv);
        }
        return cont(new Match(state, rv.match, rv.commit));
      });
    });
  };

  commit = function(p) {
    p = implicit(p);
    return new Parser((function() {
      return resolve(p).message();
    }), function(state, cont) {
      p = resolve(p);
      return p.parse(state, function(rv) {
        if (!rv.ok) {
          return cont(rv);
        }
        rv.commit = true;
        return cont(rv);
      });
    });
  };

  not_ = function(p) {
    var message;
    p = implicit(p);
    message = function() {
      return "not " + resolve(p).message();
    };
    return new Parser(message, function(state, cont) {
      p = resolve(p);
      return p.parse(state, function(rv) {
        if (rv.ok) {
          return state.fail(cont, message());
        } else {
          return cont(new Match(state, ""));
        }
      });
    });
  };

  drop = function(p) {
    return implicit(p).onMatch(function(x) {
      return null;
    });
  };

  dropPrefix = function(prefix, p) {
    return seq(optional(prefix), p).onMatch(function(x) {
      return x[1];
    });
  };

  chain = function(p1, p2, combiner) {
    return new Parser((function() {
      return "(" + (resolve(p1).message()) + ") then (" + (resolve(p2).message()) + ")";
    }), function(state, cont) {
      p1 = resolve(p1);
      return p1.parse(state, function(rv1) {
        if (!rv1.ok) {
          return cont(rv1);
        }
        p2 = resolve(p2);
        return p2.parse(rv1.state, function(rv2) {
          if (!rv2.ok) {
            if (rv1.commit) {
              rv2.abort = true;
            }
            return cont(rv2);
          }
          return cont(new Match(rv2.state, combiner(rv1.match, rv2.match), rv2.commit));
        });
      });
    });
  };

  seq = function() {
    var combiner, message, p, parsers, rv, _i, _len;
    parsers = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
    parsers = (function() {
      var _i, _len, _results;
      _results = [];
      for (_i = 0, _len = parsers.length; _i < _len; _i++) {
        p = parsers[_i];
        _results.push(implicit(p));
      }
      return _results;
    })();
    message = function() {
      return ((function() {
        var _i, _len, _results;
        _results = [];
        for (_i = 0, _len = parsers.length; _i < _len; _i++) {
          p = parsers[_i];
          _results.push("(" + resolve(p).message() + ")");
        }
        return _results;
      })()).join(" then ");
    };
    combiner = function(sum, x) {
      sum = sum.slice(0);
      if (x != null) {
        sum.push(x);
      }
      return sum;
    };
    rv = new Parser("''", function(state, cont) {
      return cont(new Match(state, []));
    });
    for (_i = 0, _len = parsers.length; _i < _len; _i++) {
      p = parsers[_i];
      rv = chain(rv, p, combiner);
    }
    return new Parser(message, function(state, cont) {
      return rv.parse(state, cont);
    });
  };

  seqIgnore = function() {
    var ignore, message, newseq, p, parsers, rv, _i, _len;
    ignore = arguments[0], parsers = 2 <= arguments.length ? __slice.call(arguments, 1) : [];
    parsers = (function() {
      var _i, _len, _results;
      _results = [];
      for (_i = 0, _len = parsers.length; _i < _len; _i++) {
        p = parsers[_i];
        _results.push(implicit(p));
      }
      return _results;
    })();
    message = function() {
      return ((function() {
        var _i, _len, _results;
        _results = [];
        for (_i = 0, _len = parsers.length; _i < _len; _i++) {
          p = parsers[_i];
          _results.push("(" + resolve(p).message() + ")");
        }
        return _results;
      })()).join(" then ");
    };
    newseq = [];
    for (_i = 0, _len = parsers.length; _i < _len; _i++) {
      p = parsers[_i];
      newseq.push(optional(ignore).drop());
      newseq.push(p);
    }
    rv = seq.apply(null, newseq);
    return new Parser(message, function(state, cont) {
      return rv.parse(state, cont);
    });
  };

  alt = function() {
    var message, p, parsers;
    parsers = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
    parsers = (function() {
      var _i, _len, _results;
      _results = [];
      for (_i = 0, _len = parsers.length; _i < _len; _i++) {
        p = parsers[_i];
        _results.push(implicit(p));
      }
      return _results;
    })();
    message = function() {
      return ((function() {
        var _i, _len, _results;
        _results = [];
        for (_i = 0, _len = parsers.length; _i < _len; _i++) {
          p = parsers[_i];
          _results.push("(" + resolve(p).message() + ")");
        }
        return _results;
      })()).join(" or ");
    };
    return new Parser(message, function(state, cont) {
      var aborting, _i, _j, _len, _len1, _ref, _results;
      parsers = (function() {
        var _i, _len, _results;
        _results = [];
        for (_i = 0, _len = parsers.length; _i < _len; _i++) {
          p = parsers[_i];
          _results.push(resolve(p));
        }
        return _results;
      })();
      if (debug != null) {
        debug("<alt> start:");
        for (_i = 0, _len = parsers.length; _i < _len; _i++) {
          p = parsers[_i];
          debug("<alt> -- " + p);
        }
        debug("<alt> --.");
      }
      aborting = false;
      _ref = parsers.reverse();
      _results = [];
      for (_j = 0, _len1 = _ref.length; _j < _len1; _j++) {
        p = _ref[_j];
        _results.push((function(p) {
          return state.trampoline.push(function() {
            if (debug != null) {
              debug("<alt> next try: " + p);
            }
            if (aborting) {
              return;
            }
            return p.parse(state, function(rv) {
              if (rv.abort) {
                aborting = true;
              }
              return cont(rv);
            });
          });
        })(p));
      }
      return _results;
    });
  };

  repeat = function(p, minCount, maxCount) {
    var countMessage, message;
    if (minCount == null) {
      minCount = 0;
    }
    if (maxCount == null) {
      maxCount = null;
    }
    p = implicit(p);
    if (maxCount != null) {
      countMessage = "{" + minCount + ", " + maxCount + "}";
    } else {
      countMessage = "{" + minCount + "+}";
      maxCount = Math.pow(2, 31);
    }
    message = function() {
      return "(" + resolve(p).message() + (")" + countMessage);
    };
    return new Parser(message, function(state, cont) {
      var count, nextCont, origState;
      p = resolve(p);
      origState = state;
      count = 0;
      nextCont = function(rv, list, lastState) {
        if (list == null) {
          list = [];
        }
        if (lastState == null) {
          lastState = origState;
        }
        if (!rv.ok) {
          if (count >= minCount) {
            return cont(new Match(lastState, list, rv.commit));
          }
          return origState.fail(cont, message());
        }
        count += 1;
        if (rv.match != null) {
          list.push(rv.match);
        }
        if (count < maxCount) {
          if (rv.state.pos === origState.pos) {
            throw new Error("Repeating parser isn't making progress: " + p);
          }
          return rv.state.trampoline.push(function() {
            return p.parse(rv.state, function(x) {
              return nextCont(x, list.slice(0), rv.state);
            });
          });
        } else {
          return cont(new Match(rv.state, list, rv.commit));
        }
      };
      return p.parse(origState, nextCont);
    });
  };

  repeatIgnore = function(ignore, p, minCount, maxCount) {
    var p2;
    if (minCount == null) {
      minCount = 0;
    }
    if (maxCount == null) {
      maxCount = null;
    }
    p2 = seq(optional(ignore).drop(), p).onMatch(function(x) {
      return x[0];
    });
    return repeat(p2, minCount, maxCount);
  };

  repeatSeparated = function(p, separator, minCount, maxCount) {
    var p2;
    if (separator == null) {
      separator = "";
    }
    if (minCount == null) {
      minCount = 1;
    }
    if (maxCount == null) {
      maxCount = null;
    }
    p2 = seq(drop(separator), p).onMatch(function(x) {
      return x[0];
    });
    return seq(p, repeat(p2, minCount - 1, maxCount != null ? maxCount - 1 : maxCount)).onMatch(function(x) {
      return [x[0]].concat(x[1]);
    });
  };

  reduce = function(p, separator, accumulator, reducer, minCount, maxCount) {
    if (separator == null) {
      separator = "";
    }
    if (accumulator == null) {
      accumulator = null;
    }
    if (reducer == null) {
      reducer = null;
    }
    if (minCount == null) {
      minCount = 1;
    }
    if (maxCount == null) {
      maxCount = null;
    }
    if (!(accumulator != null)) {
      accumulator = function(x) {
        return [x];
      };
    }
    if (!(reducer != null)) {
      reducer = function(sum, sep, x) {
        return sum.push(x);
      };
    }
    return seq(p, repeat(seq(separator, p), minCount - 1, maxCount != null ? maxCount - 1 : maxCount)).onMatch(function(x) {
      return [accumulator(x[0])].concat(x[1]).reduce(function(sum, item) {
        return reducer(sum, item[0], item[1]);
      });
    });
  };

  implicit = function(p) {
    if (typeof p === "string") {
      return string(p);
    }
    if (p instanceof RegExp) {
      return regex(p);
    }
    if (p instanceof Array) {
      return seq.apply(null, p);
    }
    return p;
  };

  resolve = function(p) {
    if (!(p instanceof Function)) {
      return implicit(p);
    }
    p = implicit(p());
    if (!(p != null)) {
      throw new Error("Can't resolve parser");
    }
    return p;
  };

  parse = function(p, str) {
    var failures, state, successes, x, _i, _j, _len, _len1;
    state = str instanceof ParserState ? str : new ParserState(str);
    p = resolve(p);
    successes = [];
    failures = [];
    state.trampoline.push(function() {
      return p.parse(state, function(rv) {
        if (rv.ok) {
          if (debug != null) {
            debug("--- registering success: " + rv);
          }
          return successes.push(rv);
        } else {
          if (debug != null) {
            debug("--- registering failure: " + rv);
          }
          return failures.push(rv);
        }
      });
    });
    while (state.trampoline.ready() && successes.length === 0) {
      state.trampoline.next();
    }
    if (debug != null) {
      debug("--- final tally:");
      for (_i = 0, _len = successes.length; _i < _len; _i++) {
        x = successes[_i];
        debug("+++ " + x);
      }
      for (_j = 0, _len1 = failures.length; _j < _len1; _j++) {
        x = failures[_j];
        debug("--- " + x);
      }
      debug("--- GOOD DAY SIR");
    }
    if (successes.length > 0) {
      return successes[0];
    } else {
      return failures[0];
    }
  };

  consume = function(p, str) {
    p = chain(implicit(p), end, function(a, b) {
      return a;
    });
    return parse(p, str);
  };

  exports.setDebug = setDebug;

  exports.ParserState = ParserState;

  exports.Match = Match;

  exports.NoMatch = NoMatch;

  exports.Parser = Parser;

  exports.end = end;

  exports.reject = reject;

  exports.string = string;

  exports.regex = regex;

  exports.optional = optional;

  exports.check = check;

  exports.commit = commit;

  exports.not_ = not_;

  exports.drop = drop;

  exports.dropPrefix = dropPrefix;

  exports.seq = seq;

  exports.seqIgnore = seqIgnore;

  exports.alt = alt;

  exports.repeat = repeat;

  exports.repeatIgnore = repeatIgnore;

  exports.repeatSeparated = repeatSeparated;

  exports.reduce = reduce;

  exports.implicit = implicit;

  exports.parse = parse;

  exports.consume = consume;

}).call(this);
